function [D,varargout]=SERIEDATE(D1,D2,varargin)
//******************************************************************
// Fonction permettant de gnrer une srie de date  pas de temps 
// fixe (journalier, mensuel ou annuel) entre une date D1 de dpart et D2 de 
// fin
//
//  Input arguments of the function :
//        D1    Date de dpart
//        D2    Date de fin
//        {opt. param. 1} PDT - Pas de temps ('h', 'j', 'm' ou 'a') , par dfaut ='m'
//        {opt. param. 2} PAS - Nb de pas de temps entre chq valeur
//        {opt. param. 3} TYPE - Type des dates D1 et D2 (0 = AAAAMMJJHHmm, 1= 36550)
//
//  Output arguments of the function :
//        D     Srie de dates
//        {opt. param. 1} Srie de date au format texte jj/mm/aa hh:mm
//

//------ Validation of input variables -------
    if size(D1,1)*size(D1,2)*size(D2,1)*size(D2,2) <> 1 then,  error('D1 or D2 are not single date values'); end

//------ Initialisation -------

    [lhs,rhs] = argn();   // The following list attributes defaults parameters values if no other values are defined

    PDT       = 'm';  if rhs >= 3  then,  PDT= varargin(1); end     // Rcupration du pas de temps
    if PDT=='m' | PDT=='j' | PDT=='a' | PDT=='h' then, ; else error('PDT is not properly defined'); end

    PAS       = 1;    if rhs >= 4  then,  PAS= varargin(2); end     // Rcupration du pas de temps
    if PAS<0 then, error('PAS is not properly defined'); end;

    TYPE      = 0;    if rhs >= 5 then,   TYPE=varargin(3);end      // Rcupration du type de D1 et D2
    if TYPE==0 | TYPE==1 then, ; else error('TYPE is not properly defined'); end


//------ Calculs -------

    // Gnration des dates D1digit, D2digit, D1num, D2num
      if TYPE==0 then
          D1digit=D1;D2digit=D2;D1num=c_CONVDATE(0,D1);D2num=c_CONVDATE(0,D2);
      elseif TYPE==1 then
          D1digit=c_CONVDATE(1,D1);D2digit=c_CONVDATE(1,D2);D1num=D1;D2num=D2;
      end

    // Gnration de la srie intermdiaire 
      if D1num>0 & D1digit>0 & D2num>0 & D2digit>0 then // Vrification de la validit des dates D1 et D2

        if PDT=='a' then // Pas annuel
          a1    = int(D1digit/10^8)-PAS;
          a2    = int(D2digit/10^8)+PAS;          
          D     = c_CONVDATE(0,(a1*10^8+1010000:PAS*10^8:a2*10^8+1010000)');

        elseif PDT=='m' then // Pas mensuel
          m1    = int(D1digit/10^6);
          m2    = int(D2digit/10^6);          
          D     = c_CONVDATE(0,(m1*10^6+10000:PAS*10^6:m2*10^6+10000)');

        elseif PDT=='j' then // Pas journalier
          D     = (int(D1num):PAS:int(D2num)+PAS)';
        
        elseif PDT=='h' then // Pas horaire
          D     = (int(D1num)+int((D1num-int(D1num))*24)/24:PAS/24:int(D2num)+(int((D2num-int(D2num))*24)+1)/24)';
        end

        D       = D(find(D>0));

        
        Ddigit  = c_CONVDATE(1,D);
        Annee   = int(Ddigit/10^8);
        Mois    = int(Ddigit/10^6)-Annee*100;
        Jour    = int(Ddigit/10^4)-Annee*10^4-Mois*100;
        Heure   = int(Ddigit/100)-Annee*10^6-Mois*10^4-Jour*100;
        Minute  = Ddigit-Annee*10^8-Mois*10^6-Jour*10^4-Heure*100;

      else
        error('Problem with conversion of D1 or D2');
      end
  
//------ Output variables---------------
      if lhs>=2 then, varargout(1)=msprintf('%2d/%2d/%2d %2d:%2d\n',Jour,Mois,Annee-int(Annee/100)*100,Heure,Minute); end

endfunction
