#include <math.h>

// Number of AR parameters
#define nbAR 1

// Max value of Qcalc
#define MAX 100000

// Max size of Qdata
#define SIZE 100000


/***********************************************
* Autoregressive model or ARMA (1,1,2)
*	AREG Xeters :
*
*	X[0] 		= Autoregressive component
*	X[nbAR]    	= Moving average component 1
*	X[nbAR+1]	= Moving average component 2
*
************************************************/

void c_AREG(long int nval,double X[],double MAXHPREV[],double Qdata[], double Qcalc[])
{
	long int i;
	int j,Deb,nprev,maxhprev;
	double err[SIZE],temp;

	if(nval>SIZE){nval=SIZE;}

	// Constants
	maxhprev = (int)(MAXHPREV[0]);
	if(maxhprev<1){maxhprev=1;}
	// Initialisation
	Deb = 2; // 1 value for the differenciation
	for( i=0 ; i < Deb; i++){
		Qcalc[i]=0;
	}
	
	// Simulation
	if (nval<Deb) {Qcalc[0]=-9.99;} // Not enough input values
	else {
		for( i=Deb ; i < nval-1; i++){
			Qcalc[i]=Qdata[i]; // No forecast

			// Forecasting error
			if(i==Deb){err[i]=0;}

			if(i>Deb+1){err[i]=Qcalc[i]-Qcalc[i-1+nval];}

			// Lead-time = 1
			Qcalc[i+nval]=Qcalc[i]+X[0]*(Qcalc[i]-Qcalc[i-1])-X[nbAR]*err[i]-X[nbAR+1]*err[i-1];

			// Lead-time = 2
			if(maxhprev>1){
				Qcalc[i+nval*2]=Qcalc[i+nval]+X[0]*(Qcalc[i+nval]-Qcalc[i])-X[nbAR+1]*err[i];
			}

			// Further lead-times
			if(maxhprev>2){
				for(j=2;j<=maxhprev;j++){
					temp = Qcalc[i+nval*(j-1)]+X[0]*(Qcalc[i+nval*(j-1)]-Qcalc[i+nval*(j-2)]);
					if(temp>MAX){Qcalc[i+nval*j]=MAX;}
					else{Qcalc[i+nval*j]=Qcalc[i+nval*(j-1)]+X[0]*(Qcalc[i+nval*(j-1)]-Qcalc[i+nval*(j-2)]);}
				}
			}

		}
		// Last line of Qcalc matrix
		//for(j=0;j<=nprev;j++){Qcalc[nval-1+nval*j]=0;}
	}
}

