#include <math.h>
#include <stdio.h>

// Number of tolerated missing data
#define NLACMAX 5

// Number of parameters
#define	npX 4


/***********************************************
* GR2M model
*	PAS = Time step (hours)	
*
*	GR2M Parameters :
*	X[0] = Soil reservoir capacity 		(> 1mm)
*	X[1] = losses parameter 		(in [-10..10])
*
************************************************/


/* ************** Core subroutine ****************************** */

void c_GR2M(long int nval,double PAS[],double Xinp[],double Pinp[],double Einp[],double Rcalc[],double Scalc[],double Qcalc[], double INI[], double ECHcalc[], double PRcalc[])
{
	int i,nLAC;
	double P,E,Q,Sini,Rini; 
	double X[4],V[2];
	double ES,PS,PR,WS,S2,PERC,ECH,R2,R1,Q2,PERCFACTOR,pas;


	/* --- Model structure ----------------- */

	//Time step
	if (PAS[0]<1) {pas=1;} 
	else {pas=PAS[0];}

	// Percolation factor
	if (pas<12) 	{PERCFACTOR=4;} 	// Hourly time steps
	if (pas>=12) {PERCFACTOR=2.25;}		// Daily time steps
	if (pas>=240) {PERCFACTOR=1;}		// Montly time steps

	//sciprint("PAS=%5.2f PERCFACTOR=%5.2f\n",PAS[0],PERCFACTOR);	


	/* --- Initialisations ----------------- */
	nLAC=0;

	// Parameter vector
	X[0]=Xinp[0];
	X[1]=Xinp[1];
	X[2]=60;		// R reservoir size is fixed
	if(X[0]<1){X[0]=1;}
	if(X[1]<0){X[1]=0;}
	if(X[1]>10){X[1]=10;}

	if(INI[0]<0){INI[0]=0;}
	if(INI[0]>1){INI[0]=1;}
	if(INI[1]<0){INI[1]=0;}
	if(INI[1]>1){INI[1]=1;}
	
	
	// State variable vector
	Sini=X[0]*INI[0]; // initial values of S and R
	Rini=X[2]*INI[1];	
	V[0]=Sini;
	V[1]=Rini;
	


	/* ------- Loop over the data -------- */
	for (i=0;i<nval;i++){
		P=Pinp[i];
		E=Einp[i];
		Q=0;

		/*--- Missing data management ---------------------------------*/ 
		if((P<0)&(nLAC<=NLACMAX)){P=0;nLAC++;}
		if((E<0)&(nLAC<=NLACMAX)){E=0;nLAC++;}
		if(nLAC>NLACMAX)         {V[0]=Sini;V[1]=Rini;P=0;E=0;nLAC=0;}

	
		/*--- CORE MODEL CODE -----------------------------------------*/ 
		
		// ......Production.............

		if(P>E)	{ES=0;
			WS=(P-E)/X[0];
			if(WS>=13){WS=13;}
			PS=X[0]*(1-pow(V[0]/X[0],2))*tanh(WS)/(1+V[0]/X[0]*tanh(WS));
			PR=P-E-PS;
			}

		else	{WS=(E-P)/X[0];
			if(WS>=13){WS=13;}
			ES=V[0]*(2-V[0]/X[0])*tanh(WS)/(1+(1-V[0]/X[0])*tanh(WS));
			PS=0;
			PR=0;
		}
		V[0]+=PS-ES;


		// ......Percolation.............

		S2=V[0]/pow(1+pow(V[0]/PERCFACTOR/X[0],3),1/3);
		PERC=V[0]-S2;
		V[0]=S2;
		PR+=PERC;
		

		// ......Routing store and Water exchange ..
				
		R1 = V[1]+PR;
		ECH=(X[1]-1)*R1;
		R2 = R1 + ECH;
		Q2 = pow(R2,2)/(R2+X[2]);
		V[1]= R2-Q2;
		
	
		/*--- RESULTS ---------------- */

		Qcalc[i]=Q2;
		Scalc[i]=V[0];
		Rcalc[i]=V[1];
		ECHcalc[i]=ECH;
		PRcalc[i]=PR;
		
		/*--- Missing data management (initial value of S and R)----------*/ 
		if(nLAC<=NLACMAX){Sini=V[0];Rini=V[1];}
	}

}
