#include <math.h>
#include "stack-c.h"

// Size of X vector (WARNING : max UH ordinates = XSIZE-5 !!)
#define XSIZE 500
#define npX 0

/***********************************************
* Simplified propagation model (lag and route with 
*   simple lag -TYPE=1x- or GR4J UH2 -TYPE=2x- and a linear -TYPE=x1- or quadratic reservoir -TYPE=x2-)
*	Parameters :
*	X[0] = Lag time
*	X[1] = Routing store reservoir size (>1)
*
************************************************/

void UHDEC (int NPX,int NH,double * X,double C)
{
	int i;
	for(i=0;i<2*NH;i++){
		X[NPX+NH+i]=0;
		if(i==(int)(C))  {X[NPX+NH+i]=(floor)(C)+1-C;}
		if(i==(int)(C)+1){X[NPX+NH+i]=C-(floor)(C);}
	}				
}

void UH2 (int NPX, int NH,double * X,double C);


void c_PROPAG(long int nval,double TYPE[],double Xinp[],double Qdata[],double Qcalc[],double Rcalc[])
{
	int i,k,NH,typeUH,typeRES;
	double Qam,Qav,R,RTP,Rsize,Lag,X[XSIZE],V[XSIZE];

	// Parameters

	typeUH=(int)(TYPE[0]/10);
	typeRES=(int)(TYPE[0])-typeUH*10;

	Lag = Xinp[0];
	if(Lag<0){Lag=0;}

	Rsize = Xinp[1];
	if((Rsize<1)&(typeRES==1)){Rsize=1;}
	if((Rsize<0)&(typeRES==2)){Rsize=0;}
	
	// Initialisation
	NH=(int)(XSIZE/3)-npX-1;
	for(i=0;i<XSIZE;i++){X[i]=0;}
	for (i=0;i<XSIZE;i++){V[i]=0;}
	
	// LAG model
	if(typeUH==1){UHDEC(npX,NH,X,Lag);}
	if(typeUH==2){UH2(npX,NH,X,Lag);}

	R=0;	

	// Simulation
	for(i=0;i<nval;i++){
		Qam=Qdata[i];	
		if(Qam<0){Qam=0;}
		
		// ......Upstream discharge UH ...........
		for (k=0;k<2*NH;k++){
			V[k]=V[1+k]+X[npX+NH+k]*Qam;
		}
		V[2*NH]=X[npX+3*NH-1]*Qam;

		// ...... Lag & Route reservoir .........		
		if(V[0]<0){V[0]=0;}
		RTP=R+V[0];
		if(RTP>=0){R=RTP;}

		//printf("%5d V=%5.2f R1=%5.2f ",i,V[0],R);

		// Reservoir output
		if(typeRES==1){Qav=R/Rsize;}
		if((typeRES==2)&(Rsize+R>0)){Qav=pow(R,2)/(R+Rsize);}
		if((typeRES==2)&(Rsize+R==0)){Qav=0;}
		if(!(typeRES==1)&!(typeRES==2)){Qav=0;}

		R-=Qav;

		Qcalc[i]=Qav;
		Rcalc[i]=R;
		
	}
}
